require "spec_helper"

class TestPage
  include SpecTest

  url_is "http://localhost:1234"

  link :static_text_examples, :text => "Static Text Examples"
  link :static_text_examples_b do |this| "testing" end
  text_field :server, :id => "server"
  text_field :server_b do |this| "testing" end
  button :submit, :id => "submit"
  button :submit_b do |content| "testing" end
end

class TestingPage
  include SpecTest
end

describe SpecTest::Generators do
  let(:watir_browser) { mock_browser_with_watir }
  let(:selenium_browser) { mock_browser_with_selenium }
  let(:watir_page) { TestPage.new(watir_browser) }
  let(:selenium_page) { TestPage.new(selenium_browser) }

  describe "there are page objects that contain location information" do
    context "which means" do
      it "it should navigate to a page when requested" do
        watir_browser.should_receive(:goto)
        page = TestPage.new(watir_browser, true)
      end

      it "it should not navigate to a page when not requested" do
        watir_browser.should_not_receive(:goto)
        page = TestPage.new(watir_browser)
      end

      it "it should not navigate to a page when a url is not specified" do
        watir_browser.should_not_receive(:goto)
        page = TestingPage.new(watir_browser, true)
      end
    end
  end

  describe "there are link generators" do
    context "and when a link is declared on a page" do
      it "methods for querying and manipulating the link should be generated" do
        watir_page.should respond_to(:static_text_examples)
        watir_page.should respond_to(:static_text_examples_object)
        watir_page.should respond_to(:static_text_examples_link)
      end

      it "blocks declared with the object will be executed" do
        watir_page.static_text_examples_b_link.should == "testing"
      end
    end

    context "and when used on the watir platform implementation" do
      it "it should be possible to find a link object" do
        watir_browser.should_receive(:link).and_return(watir_browser)
        web_object = watir_page.static_text_examples_object
        web_object.should be_instance_of SpecTest::WebObjects::Link
      end

      it "it should be possible to select (and thus click) a link" do
        watir_browser.stub_chain(:link, :click)
        watir_page.static_text_examples
      end
    end

    context "and when used on the selenium platform implementation" do
      it "it should be possible to find a link object" do
        selenium_browser.should_receive(:find_element).and_return(selenium_browser)
        web_object = selenium_page.static_text_examples_object
        web_object.should be_instance_of SpecTest::WebObjects::Link
      end

      it "it should be possible to select (and thus click) a link" do
        selenium_browser.stub_chain(:find_element, :click)
        selenium_page.static_text_examples
      end
    end
  end

  describe "there are text field generators" do
    context "and when a text field is declared on a page" do
      it "methods for querying and manipulating the text field should be generated" do
        watir_page.should respond_to(:server)
        watir_page.should respond_to(:server=)
        watir_page.should respond_to(:server_object)
        watir_page.should respond_to(:server_text_field)
      end

      it "blocks declared with the object will be executed" do
        watir_page.server_b_object.should == "testing"
      end
    end

    context "and when used on the watir platform implementation" do
      it "it should be possible to find a text field object" do
        watir_browser.should_receive(:text_field).and_return(watir_browser)
        web_object = watir_page.server_object
        web_object.should be_instance_of SpecTest::WebObjects::TextField
      end

      it "it should be possible to get the text from a text field" do
        watir_browser.should_receive(:text_field).and_return(watir_browser)
        watir_browser.should_receive(:value).and_return("Testing")
        watir_page.server.should == "Testing"
      end

      it "it should be possible to type text into a text field" do
        watir_browser.should_receive(:text_field).and_return(watir_browser)
        watir_browser.should_receive(:set).with("Testing")
        watir_page.server = "Testing"
      end
    end

    context "and when used on the selenium platform implementation" do
      it "it should be possible to find a text field object" do
        selenium_browser.should_receive(:find_element).and_return(selenium_browser)
        web_object = selenium_page.server_object
        web_object.should be_instance_of SpecTest::WebObjects::TextField
      end

      it "it should be possible to get the text from a text field" do
        selenium_browser.should_receive(:find_element).and_return(selenium_browser)
        selenium_browser.should_receive(:attribute).with('value').and_return("Testing")
        selenium_page.server.should == "Testing"
      end

      it "it should be possible to type text into a text field" do
        selenium_browser.should_receive(:find_element).twice.and_return(selenium_browser)
        selenium_browser.should_receive(:clear)
        selenium_browser.should_receive(:send_keys).with("Testing")
        selenium_page.server = "Testing"
      end
    end
  end

  describe "there are button generators" do
    context "and when a button is declared on a page" do
      it "methods for querying and manipulating the button should be generated" do
        watir_page.should respond_to(:submit)
        watir_page.should respond_to(:submit_object)
        watir_page.should respond_to(:submit_button)
      end

      it "blocks declared with the object will be executed" do
        watir_page.submit_b_object.should == "testing"
      end
    end

    context "and when used on the watir platform implementation" do
      it "it should be possible to find a button object" do
        watir_browser.should_receive(:button).and_return(watir_browser)
        web_object = watir_page.submit_object
        web_object.should be_instance_of SpecTest::WebObjects::Button
      end

      it "it should be possible to click a button object" do
        watir_browser.should_receive(:button).and_return(watir_browser)
        watir_browser.should_receive(:click)
        watir_page.submit
      end
    end

    context "and when used on the selenium platform implementation" do
      it "it should be possible to find a button object" do
        selenium_browser.should_receive(:find_element).and_return(selenium_browser)
        web_object = selenium_page.submit_object
        web_object.should be_instance_of SpecTest::WebObjects::Button
      end

      it "it should be possible to click a button object" do
        selenium_browser.should_receive(:find_element).and_return(selenium_browser)
        selenium_browser.should_receive(:click)
        selenium_page.submit
      end
    end
  end
end